#undef HID_ENABLED

// Arduino Due ADC->DMA->USB 1MSPS
// by stimmer
// from http://forum.arduino.cc/index.php?topic=137635.msg1136315#msg1136315
// Input: Analog in A0
// Output: Raw stream of uint16_t in range 0-4095 on Native USB Serial/ACM

inline void digitalWriteDirect(int pin, boolean val) {
  if (val) g_APinDescription[pin].pPort -> PIO_SODR = g_APinDescription[pin].ulPin;
  else    g_APinDescription[pin].pPort -> PIO_CODR = g_APinDescription[pin].ulPin;
}
#define NBLOCKS 4
#define BSIZ  128*NBLOCKS                 // and in python it's 4 times this factor

volatile int bdone, bufn, obufn;
uint16_t buf[4][BSIZ];   // 4 buffers
uint16_t tt;
uint8_t nblocks;

void ADC_Handler() {    // move DMA pointers to next buffer
  int f = ADC->ADC_ISR;
  if (f & (1 << 27)) {
    bufn = (bufn + 1) & 3;
    ADC->ADC_RNPR = (uint32_t)buf[bufn];
    ADC->ADC_RNCR = BSIZ;
    digitalWriteDirect(10, LOW);
    digitalWriteDirect(10, HIGH);
    if (bdone == 5)
    {
      NVIC_DisableIRQ(ADC_IRQn);
    }
    bdone++;
  }
}

void setup()
{
  SerialUSB.begin(0);
  pmc_enable_periph_clk(ID_ADC);
  //adc_init(ADC, SystemCoreClock/8, ADC_FREQ_MAX, ADC_STARTUP_FAST);   // 1Mhz sample rate
  adc_init(ADC, SystemCoreClock, ADC_FREQ_MAX, ADC_STARTUP_FAST);
  ADC->ADC_MR |= 0x80; // free running
  ADC->ADC_MR &= 0xffff00ff;
  tt = 9;         // from 0 to 255 only   sample rate = 1e6 /(1+tt)....tt=9 = 1e5 samples/sec
  ADC->ADC_MR |= (tt << 8);

  ADC->ADC_CHER = 0x80;
  NVIC_EnableIRQ(ADC_IRQn);
  ADC->ADC_IDR = ~(1 << 27);
  ADC->ADC_IER = 1 << 27;
  ADC->ADC_RPR = (uint32_t)buf[0]; // DMA buffer
  ADC->ADC_RCR = BSIZ;
  ADC->ADC_RNPR = (uint32_t)buf[1]; // next DMA buffer
  ADC->ADC_RNCR = BSIZ;
  ADC->ADC_COR = 0x030000;        // A0 now diff mode. A1+, A0-, result in A0 register
  bufn = 1;
  obufn = 0;
  ADC->ADC_PTCR = 1;
  ADC->ADC_CR = 2;
  pinMode(8, OUTPUT);
  pinMode(9, OUTPUT);
  pinMode(10, OUTPUT);
}

void loop()
{
  int c;
  uint16_t n;
  if (SerialUSB.available() > 0)
  {
    c = SerialUSB.read();
    if (c == 'a')      tt = 199;
    if (c == 'b')      tt = 39;
    if (c == 'c')      tt = 9;
    if (c == 'e')      tt = 1;
    if (c == 't')
    {
      digitalWriteDirect(9, HIGH);
      SerialUSB.println("test");
      digitalWriteDirect(9, LOW);
    }
    if (c == 'p')
      SerialUSB.write(tt);
    if (c == 'n')
      SerialUSB.write(nblocks);
    if (c == 'd')
    {
      bufn = 0;
      bdone = 0;
      NVIC_EnableIRQ(ADC_IRQn);
      digitalWriteDirect(9, HIGH);
      while (bdone != 5)
        ;
      digitalWriteDirect(9, LOW);
      SerialUSB.write((uint8_t *)buf[1], BSIZ * 2);
      SerialUSB.write((uint8_t *)buf[2], BSIZ * 2);
      SerialUSB.write((uint8_t *)buf[3], BSIZ * 2);
      SerialUSB.write((uint8_t *)buf[0], BSIZ * 2);
 
  pmc_disable_periph_clk(ID_ADC);
  NVIC_DisableIRQ(ADC_IRQn);
  adc_init(ADC, SystemCoreClock, ADC_FREQ_MAX, ADC_STARTUP_FAST);
  ADC->ADC_MR |= 0x80; // free running
  ADC->ADC_MR &= 0xffff00ff;
  ADC->ADC_MR |= (tt << 8);
  ADC->ADC_CHER = 0x80;
  NVIC_EnableIRQ(ADC_IRQn);
  ADC->ADC_IDR = ~(1 << 27);
  ADC->ADC_IER = 1 << 27;
  ADC->ADC_RPR = (uint32_t)buf[0]; // DMA buffer
  ADC->ADC_RCR = BSIZ;
  ADC->ADC_RNPR = (uint32_t)buf[1]; // next DMA buffer
  ADC->ADC_RNCR = BSIZ;
  ADC->ADC_COR = 0x030000;        // A0 now diff mode. A1+, A0-, result in A0 register

  ADC->ADC_PTCR = 1;
  ADC->ADC_CR = 2;
  bufn = 0;
  bdone = 0;
    pmc_enable_periph_clk(ID_ADC);
    }
  }
}
